package user

import (
	"errors"
	"gitea.pena/PenaSide/common/log_mw"
	"gitea.pena/PenaSide/verification/internal/client"
	"gitea.pena/PenaSide/verification/internal/models"
	"gitea.pena/PenaSide/verification/internal/repository"
	"gitea.pena/PenaSide/verification/pkg/validate_controllers"
	"github.com/gofiber/fiber/v2"
	"github.com/valyala/fasthttp"
	"mime/multipart"
)

type VerifyUserControllerDeps struct {
	Repository *repository.VerificationRepository
	Telegram   *client.Telegram
}

type VerifyUserController struct {
	repository *repository.VerificationRepository
	telegram   *client.Telegram
}

func NewVerificationUserController(deps VerifyUserControllerDeps) *VerifyUserController {
	return &VerifyUserController{
		repository: deps.Repository,
		telegram:   deps.Telegram,
	}
}

func (r *VerifyUserController) GetVerification(c *fiber.Ctx) error {
	userID := c.Locals(models.UserCtxKey).(string)
	if userID == "" {
		return fiber.NewError(fiber.StatusUnauthorized)
	}

	resp, err := r.repository.GetByUserID(c.Context(), userID)
	if err != nil {
		return c.Status(fiber.StatusInternalServerError).SendString(err.Error())
	}

	if resp == nil {
		return c.SendStatus(fiber.StatusNotFound)
	}

	return c.Status(fiber.StatusOK).JSON(resp)
}

func (r *VerifyUserController) CreateVerification(c *fiber.Ctx) error {
	var req models.ReqCreateVerification
	baseURL := c.BaseURL()
	hloger := log_mw.ExtractLogger(c)

	userID := c.Locals(models.UserCtxKey).(string)
	if userID == "" {
		return fiber.NewError(fiber.StatusUnauthorized)
	}

	existingVerification, err := r.repository.GetByUserID(c.Context(), userID)
	if err != nil {
		return fiber.NewError(fiber.StatusInternalServerError, err.Error())
	}

	// чекаем только если создание уже было
	if existingVerification != nil {
		// если уже апрув, то 208
		if existingVerification.Accepted {
			return fiber.NewError(fiber.StatusAlreadyReported, "verification already completed")
		}
		// если не апрувнуто и при этом коммент пуст, то значит админ еще не добрался - 208
		if !existingVerification.Accepted && existingVerification.Comment == "" {
			return fiber.NewError(fiber.StatusAlreadyReported, "verification in progress")
		}
	}

	err = c.BodyParser(&req)
	if err != nil {
		return fiber.NewError(fiber.StatusBadRequest, err.Error())
	}

	errValidate := validate_controllers.ValidateStruct(&req)
	if errValidate != nil {
		return c.Status(fiber.StatusBadRequest).JSON(errValidate)
	}

	// INN FILE
	innFH, err := c.FormFile("inn")

	if err != nil || innFH.Size == 0 || innFH == nil {
		return fiber.NewError(fiber.StatusBadRequest, "inn file required")
	}

	// RULE FILE
	ruleFH, err := c.FormFile("rule")

	if err != nil || ruleFH.Size == 0 || ruleFH == nil {
		return fiber.NewError(fiber.StatusBadRequest, "rule file required")
	}

	// CERTIFICATE FILE
	var certFH *multipart.FileHeader
	if req.Status == "nko" {
		certFH, err = c.FormFile("certificate")
		if err != nil || certFH.Size == 0 || certFH == nil {
			return fiber.NewError(fiber.StatusBadRequest, "certificate file required")
		}
	} else {
		certFH = nil
	}

	result, err := r.repository.Insert(c.Context(), userID, &models.Verification{
		UserID:   userID,
		Accepted: false,
		Status:   req.Status,
		Comment:  "",
	}, innFH, ruleFH, certFH)
	if err != nil {
		return fiber.NewError(fiber.StatusInternalServerError, err.Error())
	}

	err = r.telegram.SendVerification(result, baseURL, false)
	if err != nil {
		return fiber.NewError(fiber.StatusInternalServerError, err.Error())
	}

	hloger.Emit(models.InfoVerificationRequested{
		CtxID:     result.ID,
		KeyStatus: result.Status,
	})

	return c.Status(fiber.StatusOK).JSON(result)
}

func (r *VerifyUserController) SetVerificationFile(c *fiber.Ctx) error {
	baseURL := c.BaseURL()
	hloger := log_mw.ExtractLogger(c)

	userID := c.Locals(models.UserCtxKey).(string)
	if userID == "" {
		return fiber.NewError(fiber.StatusUnauthorized)
	}

	existingVerification, err := r.repository.GetByUserID(c.Context(), userID)
	if err != nil {
		return fiber.NewError(fiber.StatusInternalServerError, err.Error())
	}

	// если не создавалась, то говорим что надо создать
	if existingVerification == nil {
		return fiber.NewError(fiber.StatusNotFound, "verification not initiated")
	}
	// если уже апрув, запрещаем добавлять еще файлы 208
	if existingVerification.Accepted {
		return fiber.NewError(fiber.StatusAlreadyReported, "verification already completed")
	}
	// если реджект и коммент пуст, значит админ не добрался еще, запрещаем обновлять файлы и отправляем 208
	if !existingVerification.Accepted && existingVerification.Comment == "" {
		return fiber.NewError(fiber.StatusAlreadyReported, "verification in progress, cannot update files")
	}

	availableFiles := []string{"inn", "rule", "certificate"}
	var fileHeader *multipart.FileHeader
	var result *models.Verification

	for _, fileName := range availableFiles {
		fileHeader, err = c.FormFile(fileName)

		if errors.Is(err, fasthttp.ErrMissingFile) {
			continue
		}

		if err != nil {
			return fiber.NewError(fiber.StatusInternalServerError, err.Error())
		}

		result, err = r.repository.UpdateFile(c.Context(), userID, fileName, fileHeader)
		if err != nil {
			return fiber.NewError(fiber.StatusInternalServerError, err.Error())
		}
	}

	err = r.telegram.SendVerification(result, baseURL, true)
	if err != nil {
		return fiber.NewError(fiber.StatusInternalServerError, err.Error())
	}

	hloger.Emit(models.InfoVerificationRequested{
		CtxID:     result.ID,
		KeyStatus: result.Status,
	})

	return c.SendStatus(fiber.StatusOK)
}
